let oauth2Server = require('oauth2-server');
let AuthService = require('../services/auth');
let responseService = require('../services/response');
let jwt = require('jsonwebtoken');

function AuthController(app) {

    let Request = oauth2Server.Request;
    let Response = oauth2Server.Response;
    let authService = new AuthService(app);

    /**
     * Validates user and sends a OAuth2 token
     * @param {Request} req Node request
     * @param {Response} res Node resposne
     */
    this.loginWithOAuth2 = async (req, res) => {
        let request = getLoginRequest(req);
        let response = new Response(res);
        try {
            await app.oauth.token(request, response);
            let token = jwt.sign({ user: req.body.username }, authService.getSecret());
            res.json({ token: token });
        } catch (err) {
            responseService.error(res, err.status, err.code, err.message);
        }
    };

    /**
     * Validates user and send a json web token
     * @param {Request} req Node request
     * @param {Response} res Node response
     */
    this.loginWithJWT = async (req, res) => {
        if (authService.authUserJWT(req.body.username, req.body.password)) {
            let user = await authService.getRandomUser();
            let token = jwt.sign({ email: user.email, id: user.login.uuid }, authService.getSecret());
            authService.setLoginAsJWT();
            res.json({ data: Object.assign(user, { token }) });
        } else {
            responseService.throwNotFound(res);
        }
    };

    /**
     * Retrieves an OAuth2 request with specified client
     * @param {Request} req Node request
     * @returns {Request} OAuth2 request
     */
    let getLoginRequest = (req) => {
        req.body.username = 'zeedhi@teknisa.com';
        req.body.password = 'zeedhi';
        return new Request(req);
    };
}

module = module.exports = AuthController;
